@@ -194,7 +194,7 @@ RestXmlNode *ovirt_proxy_get_collection_xml(OvirtProxy *proxy,
 
 typedef struct {
     OvirtProxy *proxy;
-    GSimpleAsyncResult *result;
+    GTask *task;
     GCancellable *cancellable;
     gulong cancellable_cb_id;
     OvirtProxyCallAsyncCb call_async_cb;
@@ -204,28 +204,26 @@ typedef struct {
 
 static void ovirt_proxy_call_async_data_free(OvirtProxyCallAsyncData *data)
 {
-        if (data->destroy_call_data != NULL) {
-            data->destroy_call_data(data->call_user_data);
+    if (data->destroy_call_data != NULL) {
+        data->destroy_call_data(data->call_user_data);
+    }
+
+    g_clear_object(&data->proxy);
+    g_clear_object(&data->task);
+
+    if ((data->cancellable != NULL) && (data->cancellable_cb_id != 0)) {
+        if (g_cancellable_is_cancelled(data->cancellable)) {
+            /* Cancellable has already been cancelled, we don't need to use
+             * g_cancellable_disconnect() to disconnect the signal handler
+             * as we know the 'cancelled' signal is no longer going to be emitted
+             */
+            g_signal_handler_disconnect(data->cancellable, data->cancellable_cb_id);
+        } else {
+            g_cancellable_disconnect(data->cancellable, data->cancellable_cb_id);
         }
-        if (data->proxy != NULL) {
-            g_object_unref(G_OBJECT(data->proxy));
-        }
-        if (data->result != NULL) {
-            g_object_unref(G_OBJECT(data->result));
-        }
-        if ((data->cancellable != NULL) && (data->cancellable_cb_id != 0)) {
-            if (g_cancellable_is_cancelled(data->cancellable)) {
-                /* Cancellable has already been cancelled, we don't need to use
-                 * g_cancellable_disconnect() to disconnect the signal handler
-                 * as we know the 'cancelled' signal is no longer going to be emitted
-                 */
-                g_signal_handler_disconnect(data->cancellable, data->cancellable_cb_id);
-            } else {
-                g_cancellable_disconnect(data->cancellable, data->cancellable_cb_id);
-            }
-        }
-        g_clear_object(&data->cancellable);
-        g_slice_free(OvirtProxyCallAsyncData, data);
+    }
+    g_clear_object(&data->cancellable);
+    g_slice_free(OvirtProxyCallAsyncData, data);
 }
 
 static void
@@ -236,17 +234,17 @@ call_async_cancelled_cb (G_GNUC_UNUSED GCancellable *cancellable,
 }
 
 
-static void rest_call_async_set_error(RestProxyCall *call, GSimpleAsyncResult *result, const GError *error)
+static void rest_call_async_set_error(RestProxyCall *call, GTask *task, const GError *error)
 {
     GError *local_error = NULL;
     RestXmlNode *root = ovirt_rest_xml_node_from_call(call);
 
     if (root != NULL && ovirt_utils_gerror_from_xml_fault(root, &local_error)) {
         g_debug("ovirt_rest_call_async(): %s", local_error->message);
-        g_simple_async_result_set_from_error(result, local_error);
+        g_task_return_error(task, local_error);
         g_clear_error(&local_error);
     } else {
-        g_simple_async_result_set_from_error(result, error);
+        g_task_return_error(task, (GError *) error);
     }
 
     if (root != NULL) {
@@ -260,33 +258,34 @@ call_async_cb(RestProxyCall *call, const GError *error,
               gpointer user_data)
 {
     OvirtProxyCallAsyncData *data = user_data;
-    GSimpleAsyncResult *result = data->result;
+    GTask *task = data->task;
+    gboolean callback_result = TRUE;
 
     if (error != NULL) {
-        rest_call_async_set_error(call, result, error);
-    } else {
-        GError *call_error = NULL;
-        gboolean callback_result = TRUE;
-
-        if (data->call_async_cb != NULL) {
-            callback_result = data->call_async_cb(data->proxy, call,
-                                                  data->call_user_data,
-                                                  &call_error);
-            if (call_error != NULL) {
-                rest_call_async_set_error(call, result, call_error);
-            }
-        }
-
-        g_simple_async_result_set_op_res_gboolean(result, callback_result);
+        rest_call_async_set_error(call, task, error);
+        goto exit;
     }
 
-    g_simple_async_result_complete (result);
+    if (data->call_async_cb != NULL) {
+        GError *call_error = NULL;
+        callback_result = data->call_async_cb(data->proxy, call,
+                                              data->call_user_data,
+                                              &call_error);
+        if (call_error != NULL) {
+            rest_call_async_set_error(call, task, error);
+            goto exit;
+        }
+    }
+
+    g_task_return_boolean(task, callback_result);
+
+exit:
     ovirt_proxy_call_async_data_free(data);
 }
 
 
 void ovirt_rest_call_async(OvirtRestCall *call,
-                           GSimpleAsyncResult *result,
+                           GTask *task,
                            GCancellable *cancellable,
                            OvirtProxyCallAsyncCb callback,
                            gpointer user_data,
@@ -302,7 +301,7 @@ void ovirt_rest_call_async(OvirtRestCall *call,
 
     data = g_slice_new0(OvirtProxyCallAsyncData);
     data->proxy = proxy;
-    data->result = result;
+    data->task = task;
     data->call_async_cb = callback;
     data->call_user_data = user_data;
     data->destroy_call_data = destroy_func;
@@ -316,8 +315,7 @@ void ovirt_rest_call_async(OvirtRestCall *call,
     if (!rest_proxy_call_async(REST_PROXY_CALL(call), call_async_cb, NULL,
                                data, &error)) {
         g_warning("Error while getting collection XML");
-        g_simple_async_result_set_from_error(result, error);
-        g_simple_async_result_complete(result);
+        g_task_return_error(task, error);
         ovirt_proxy_call_async_data_free(data);
     }
 }
@@ -325,13 +323,7 @@ void ovirt_rest_call_async(OvirtRestCall *call,
 
 gboolean ovirt_rest_call_finish(GAsyncResult *result, GError **err)
 {
-    GSimpleAsyncResult *simple;
-
-    simple = G_SIMPLE_ASYNC_RESULT(result);
-    if (g_simple_async_result_propagate_error(simple, err))
-        return FALSE;
-
-    return g_simple_async_result_get_op_res_gboolean(simple);
+    return g_task_propagate_boolean(G_TASK(result), err);
 }
 
 typedef struct {
@@ -387,7 +379,7 @@ end:
  */
 void ovirt_proxy_get_collection_xml_async(OvirtProxy *proxy,
                                           const char *href,
-                                          GSimpleAsyncResult *result,
+                                          GTask *task,
                                           GCancellable *cancellable,
                                           OvirtProxyGetCollectionAsyncCb callback,
                                           gpointer user_data,
@@ -403,7 +395,7 @@ void ovirt_proxy_get_collection_xml_async(OvirtProxy *proxy,
 
     call = ovirt_rest_call_new(proxy, "GET", href);
 
-    ovirt_rest_call_async(OVIRT_REST_CALL(call), result, cancellable,
+    ovirt_rest_call_async(OVIRT_REST_CALL(call), task, cancellable,
                           get_collection_xml_async_cb, data,
                           (GDestroyNotify)ovirt_proxy_get_collection_async_data_destroy);
     g_object_unref(call);
@@ -513,12 +505,8 @@ static char *write_to_tmp_file(const char *template,
     result = g_file_get_path(tmp_file);
 
 end:
-    if (tmp_file != NULL) {
-        g_object_unref(G_OBJECT(tmp_file));
-    }
-    if (iostream != NULL) {
-        g_object_unref(G_OBJECT(iostream));
-    }
+    g_clear_object(&tmp_file);
+    g_clear_object(&iostream);
 
     return result;
 }
@@ -576,6 +564,14 @@ static void ovirt_proxy_update_vm_display_ca(OvirtProxy *proxy)
 
         g_object_get(G_OBJECT(vm), "display", &display, NULL);
         if (display != NULL) {
+            GByteArray *ca_cert = NULL;
+            g_object_get(G_OBJECT(display), "ca-cert", &ca_cert, NULL);
+            if (ca_cert != NULL) {
+                g_byte_array_unref(ca_cert);
+                g_object_unref(display);
+                continue;
+            }
+
             g_object_set(G_OBJECT(display),
                          "ca-cert", proxy->priv->display_ca,
                          NULL);
@@ -644,8 +640,7 @@ gboolean ovirt_proxy_fetch_ca_certificate(OvirtProxy *proxy, GError **error)
     set_display_ca_cert_from_data(proxy, cert_data, cert_length);
 
 error:
-    if (source != NULL)
-        g_object_unref(source);
+    g_clear_object(&source);
 
     return load_ok;
 }
@@ -654,33 +649,32 @@ static void ca_file_loaded_cb(GObject *source_object,
                               GAsyncResult *res,
                               gpointer user_data)
 {
-    GSimpleAsyncResult *fetch_result;
+    GTask *task;
     GObject *proxy;
     GError *error = NULL;
     char *cert_data;
     gsize cert_length;
 
-    fetch_result = G_SIMPLE_ASYNC_RESULT(user_data);
+    task = G_TASK(user_data);
     g_file_load_contents_finish(G_FILE(source_object), res,
                                 &cert_data, &cert_length,
                                 NULL, &error);
     if (error != NULL) {
-        g_simple_async_result_take_error(fetch_result, error);
+        g_task_return_error(task, error);
         goto end;
     }
 
-    proxy = g_async_result_get_source_object(G_ASYNC_RESULT(fetch_result));
+    proxy = g_async_result_get_source_object(G_ASYNC_RESULT(task));
 
     set_ca_cert_from_data(OVIRT_PROXY(proxy), cert_data, cert_length);
     /* takes ownership of cert_data */
     set_display_ca_cert_from_data(OVIRT_PROXY(proxy),
                                   cert_data, cert_length);
     g_object_unref(proxy);
-    g_simple_async_result_set_op_res_gboolean(fetch_result, TRUE);
+    g_task_return_boolean(task, TRUE);
 
 end:
-    g_simple_async_result_complete (fetch_result);
-    g_object_unref(fetch_result);
+    g_object_unref(task);
 }
 
 void ovirt_proxy_fetch_ca_certificate_async(OvirtProxy *proxy,
@@ -689,7 +683,7 @@ void ovirt_proxy_fetch_ca_certificate_async(OvirtProxy *proxy,
                                             gpointer user_data)
 {
     GFile *ca_file;
-    GSimpleAsyncResult *result;
+    GTask *task;
 
     g_return_if_fail(OVIRT_IS_PROXY(proxy));
     g_return_if_fail((cancellable == NULL) || G_IS_CANCELLABLE(cancellable));
@@ -697,11 +691,12 @@ void ovirt_proxy_fetch_ca_certificate_async(OvirtProxy *proxy,
     ca_file = get_ca_cert_file(proxy);
     g_return_if_fail(ca_file != NULL);
 
-    result = g_simple_async_result_new(G_OBJECT(proxy), callback,
-                                       user_data,
-                                       ovirt_proxy_fetch_ca_certificate_async);
+    task = g_task_new(G_OBJECT(proxy),
+		              cancellable,
+		              callback,
+		              user_data);
 
-    g_file_load_contents_async(ca_file, cancellable, ca_file_loaded_cb, result);
+    g_file_load_contents_async(ca_file, cancellable, ca_file_loaded_cb, task);
     g_object_unref(ca_file);
 }
 
@@ -715,12 +710,10 @@ GByteArray *ovirt_proxy_fetch_ca_certificate_finish(OvirtProxy *proxy,
                                                     GError **err)
 {
     g_return_val_if_fail(OVIRT_IS_PROXY(proxy), NULL);
-    g_return_val_if_fail(g_simple_async_result_is_valid(result, G_OBJECT(proxy),
-                                                        ovirt_proxy_fetch_ca_certificate_async),
-                         NULL);
+    g_return_val_if_fail(g_task_is_valid(G_TASK(result), proxy), NULL);
     g_return_val_if_fail(err == NULL || *err == NULL, NULL);
 
-    if (g_simple_async_result_propagate_error(G_SIMPLE_ASYNC_RESULT(result), err))
+    if (g_task_had_error(G_TASK(result)))
         return NULL;
 
     return get_ca_cert_data(proxy);
@@ -837,24 +830,10 @@ ovirt_proxy_dispose(GObject *obj)
 {
     OvirtProxy *proxy = OVIRT_PROXY(obj);
 
-    if (proxy->priv->cookie_jar) {
-        g_object_unref(G_OBJECT(proxy->priv->cookie_jar));
-        proxy->priv->cookie_jar = NULL;
-    }
-
-    g_warn_if_fail(proxy->priv->additional_headers != NULL);
-    g_hash_table_unref(proxy->priv->additional_headers);
-    proxy->priv->additional_headers = NULL;
-
-    if (proxy->priv->api != NULL) {
-        g_object_unref(proxy->priv->api);
-        proxy->priv->api = NULL;
-    }
-
-    if (proxy->priv->display_ca != NULL) {
-        g_byte_array_unref(proxy->priv->display_ca);
-        proxy->priv->display_ca = NULL;
-    }
+    g_clear_object(&proxy->priv->cookie_jar);
+    g_clear_pointer(&proxy->priv->additional_headers, g_hash_table_unref);
+    g_clear_object(&proxy->priv->api);
+    g_clear_pointer(&proxy->priv->display_ca, g_byte_array_unref);
 
     G_OBJECT_CLASS(ovirt_proxy_parent_class)->dispose(obj);
 }
@@ -881,7 +860,6 @@ static void ovirt_proxy_constructed(GObject *gobject)
     /* FIXME: to set or not to set ?? */
     ovirt_proxy_add_header(OVIRT_PROXY(gobject), "All-Content", "true");
     ovirt_proxy_add_header(OVIRT_PROXY(gobject), "Prefer", "persistent-auth");
-    ovirt_proxy_add_header(OVIRT_PROXY(gobject), "Version", "3");
 
     /* Chain up to the parent class */
     if (G_OBJECT_CLASS(ovirt_proxy_parent_class)->constructed)
@@ -1164,9 +1142,7 @@ static void ovirt_proxy_set_api_from_xml(OvirtProxy *proxy,
 {
     OvirtCollection *vms;
 
-    if (proxy->priv->api != NULL) {
-        g_object_unref(G_OBJECT(proxy->priv->api));
-    }
+    g_clear_object(&proxy->priv->api);
     proxy->priv->api = ovirt_api_new_from_xml(node, error);
 
     vms = ovirt_api_get_vms(proxy->priv->api);
@@ -1224,15 +1200,16 @@ void ovirt_proxy_fetch_api_async(OvirtProxy *proxy,
                                  GAsyncReadyCallback callback,
                                  gpointer user_data)
 {
-    GSimpleAsyncResult *result;
+    GTask *task;
 
     g_return_if_fail(OVIRT_IS_PROXY(proxy));
     g_return_if_fail((cancellable == NULL) || G_IS_CANCELLABLE(cancellable));
 
-    result = g_simple_async_result_new (G_OBJECT(proxy), callback,
-                                        user_data,
-                                        ovirt_proxy_fetch_api_async);
-    ovirt_proxy_get_collection_xml_async(proxy, "/ovirt-engine/api", result, cancellable,
+    task = g_task_new(G_OBJECT(proxy),
+		              cancellable,
+		              callback,
+		              user_data);
+    ovirt_proxy_get_collection_xml_async(proxy, "/ovirt-engine/api", task, cancellable,
                                          fetch_api_async_cb, NULL, NULL);
 }
 
@@ -1251,11 +1228,9 @@ ovirt_proxy_fetch_api_finish(OvirtProxy *proxy,
                              GError **err)
 {
     g_return_val_if_fail(OVIRT_IS_PROXY(proxy), NULL);
-    g_return_val_if_fail(g_simple_async_result_is_valid(result, G_OBJECT(proxy),
-                                                        ovirt_proxy_fetch_api_async),
-                         NULL);
+    g_return_val_if_fail(g_task_is_valid(G_TASK(result), proxy), NULL);
 
-    if (g_simple_async_result_propagate_error(G_SIMPLE_ASYNC_RESULT(result), err))
+    if (g_task_had_error(G_TASK(result)))
         return NULL;
 
     return proxy->priv->api;
