@@ -28,20 +28,16 @@
 #include <libsoup/soup.h>
 #include <rest/rest-params.h>
 
-#include "glib-compat.h"
 #include "ovirt-proxy.h"
 #include "ovirt-resource-private.h"
 #include "ovirt-resource-rest-call.h"
 #include "ovirt-rest-call-error.h"
 #include "ovirt-utils.h"
 
-#define OVIRT_RESOURCE_REST_CALL_GET_PRIVATE(obj)                         \
-        (G_TYPE_INSTANCE_GET_PRIVATE((obj), OVIRT_TYPE_RESOURCE_REST_CALL, OvirtResourceRestCallPrivate))
-
 struct _OvirtResourceRestCallPrivate {
     OvirtResource *resource;
 } ;
-G_DEFINE_TYPE(OvirtResourceRestCall, ovirt_resource_rest_call, OVIRT_TYPE_REST_CALL);
+G_DEFINE_TYPE_WITH_PRIVATE(OvirtResourceRestCall, ovirt_resource_rest_call, OVIRT_TYPE_REST_CALL);
 
 enum {
     PROP_0,
@@ -167,8 +163,6 @@ static void ovirt_resource_rest_call_class_init(OvirtResourceRestCallClass *klas
     GParamSpec *param_spec;
     GObjectClass *object_class = G_OBJECT_CLASS(klass);
 
-    g_type_class_add_private(klass, sizeof(OvirtResourceRestCallPrivate));
-
     object_class->dispose = ovirt_resource_rest_call_dispose;
     object_class->get_property = ovirt_resource_rest_call_get_property;
     object_class->set_property = ovirt_resource_rest_call_set_property;
@@ -187,7 +181,7 @@ static void ovirt_resource_rest_call_class_init(OvirtResourceRestCallClass *klas
 
 static void ovirt_resource_rest_call_init(OvirtResourceRestCall *call)
 {
-    call->priv = OVIRT_RESOURCE_REST_CALL_GET_PRIVATE(call);
+    call->priv = ovirt_resource_rest_call_get_instance_private(call);
 }
 
 OvirtResourceRestCall *ovirt_resource_rest_call_new(RestProxy *proxy,
