@@ -141,14 +141,9 @@ static void ovirt_resource_dispose(GObject *object)
 {
     OvirtResource *resource = OVIRT_RESOURCE(object);
 
-    if (resource->priv->actions != NULL) {
-        g_hash_table_unref(resource->priv->actions);
-        resource->priv->actions = NULL;
-    }
-    if (resource->priv->sub_collections != NULL) {
-        g_hash_table_unref(resource->priv->sub_collections);
-        resource->priv->sub_collections = NULL;
-    }
+    g_clear_pointer(&resource->priv->actions, g_hash_table_unref);
+    g_clear_pointer(&resource->priv->sub_collections, g_hash_table_unref);
+
     if (resource->priv->xml != NULL) {
         g_boxed_free(REST_TYPE_XML_NODE, resource->priv->xml);
         resource->priv->xml = NULL;
@@ -571,20 +566,21 @@ void ovirt_resource_update_async(OvirtResource *resource,
                                  GAsyncReadyCallback callback,
                                  gpointer user_data)
 {
-    GSimpleAsyncResult *result;
+    GTask *task;
     OvirtResourceRestCall *call;
 
     g_return_if_fail(OVIRT_IS_RESOURCE(resource));
     g_return_if_fail(OVIRT_IS_PROXY(proxy));
     g_return_if_fail((cancellable == NULL) || G_IS_CANCELLABLE(cancellable));
 
-    result = g_simple_async_result_new(G_OBJECT(resource), callback,
-                                       user_data,
-                                       ovirt_resource_update_async);
+    task = g_task_new(G_OBJECT(resource),
+                      cancellable,
+                      callback,
+                      user_data);
 
     call = ovirt_resource_rest_call_new(REST_PROXY(proxy), resource);
     rest_proxy_call_set_method(REST_PROXY_CALL(call), "PUT");
-    ovirt_rest_call_async(OVIRT_REST_CALL(call), result, cancellable,
+    ovirt_rest_call_async(OVIRT_REST_CALL(call), task, cancellable,
                           ovirt_resource_update_async_cb, NULL, NULL);
     g_object_unref(G_OBJECT(call));
 }
@@ -595,8 +591,7 @@ gboolean ovirt_resource_update_finish(OvirtResource *resource,
                                       GError **err)
 {
     g_return_val_if_fail(OVIRT_IS_RESOURCE(resource), FALSE);
-    g_return_val_if_fail(g_simple_async_result_is_valid(result, G_OBJECT(resource),
-                                                        ovirt_resource_update_async),
+    g_return_val_if_fail(g_task_is_valid(G_TASK(result), G_OBJECT(resource)),
                          FALSE);
     g_return_val_if_fail((err == NULL) || (*err == NULL), FALSE);
 
@@ -621,19 +616,19 @@ ovirt_resource_create_rest_call_for_action(OvirtResource *resource,
                                            OvirtProxy *proxy,
                                            const char *action)
 {
-    OvirtActionRestCall *call;
+    RestProxyCall *call;
     const char *function;
 
     function = ovirt_resource_get_action(resource, action);
     g_return_val_if_fail(function != NULL, NULL);
 
-    call = ovirt_action_rest_call_new(REST_PROXY(proxy));
-    rest_proxy_call_set_method(REST_PROXY_CALL(call), "POST");
-    rest_proxy_call_set_function(REST_PROXY_CALL(call), function);
-    rest_proxy_call_add_param(REST_PROXY_CALL(call),
-                              "async", "false");
+    call = REST_PROXY_CALL(ovirt_action_rest_call_new(REST_PROXY(proxy)));
+    rest_proxy_call_set_method(call, "POST");
+    rest_proxy_call_set_function(call, function);
+    rest_proxy_call_add_param(call, "async", "false");
+    ovirt_resource_add_rest_params(resource, call);
 
-    return REST_PROXY_CALL(call);
+    return call;
 }
 
 gboolean
@@ -682,7 +677,6 @@ static enum OvirtResponseStatus parse_action_status(RestXmlNode *root,
 {
     RestXmlNode *node;
     const char *status_key = g_intern_string("status");
-    const char *state_key = g_intern_string("state");
 
     g_return_val_if_fail(g_strcmp0(root->name, "action") == 0,
                          OVIRT_RESPONSE_UNKNOWN);
@@ -695,12 +689,6 @@ static enum OvirtResponseStatus parse_action_status(RestXmlNode *root,
                     _("Could not find 'status' node"));
         g_return_val_if_reached(OVIRT_RESPONSE_UNKNOWN);
     }
-    node = g_hash_table_lookup(node->children, state_key);
-    if (node == NULL) {
-        g_set_error(error, OVIRT_ERROR, OVIRT_ERROR_PARSING_FAILED,
-                    _("Could not find 'state' node"));
-        g_return_val_if_reached(OVIRT_RESPONSE_UNKNOWN);
-    }
     g_debug("State: %s\n", node->content);
     if (g_strcmp0(node->content, "complete") == 0) {
         return OVIRT_RESPONSE_COMPLETE;
@@ -815,7 +803,7 @@ ovirt_resource_invoke_action_async(OvirtResource *resource,
                                    gpointer user_data)
 {
     RestProxyCall *call;
-    GSimpleAsyncResult *result;
+    GTask *task;
     OvirtResourceInvokeActionData *data;
 
     g_return_if_fail(OVIRT_IS_RESOURCE(resource));
@@ -829,14 +817,15 @@ ovirt_resource_invoke_action_async(OvirtResource *resource,
 						      action);
     g_return_if_fail(call != NULL);
 
-    result = g_simple_async_result_new(G_OBJECT(resource), callback,
-                                       user_data,
-                                       ovirt_resource_invoke_action_async);
+    task = g_task_new(G_OBJECT(resource),
+                      cancellable,
+                      callback,
+                      user_data);
     data = g_slice_new(OvirtResourceInvokeActionData);
     data->resource = resource;
     data->parser = response_parser;
 
-    ovirt_rest_call_async(OVIRT_REST_CALL(call), result, cancellable,
+    ovirt_rest_call_async(OVIRT_REST_CALL(call), task, cancellable,
                           ovirt_resource_invoke_action_async_cb, data,
                           (GDestroyNotify)ovirt_resource_invoke_action_data_free);
     g_object_unref(G_OBJECT(call));
@@ -849,8 +838,7 @@ ovirt_resource_action_finish(OvirtResource *resource,
                              GError **err)
 {
     g_return_val_if_fail(OVIRT_IS_RESOURCE(resource), FALSE);
-    g_return_val_if_fail(g_simple_async_result_is_valid(result, G_OBJECT(resource),
-                                                        ovirt_resource_invoke_action_async),
+    g_return_val_if_fail(g_task_is_valid(G_TASK(result), G_OBJECT(resource)),
                          FALSE);
 
     return ovirt_rest_call_finish(result, err);
@@ -892,22 +880,23 @@ void ovirt_resource_refresh_async(OvirtResource *resource,
                                   gpointer user_data)
 {
     OvirtResourceRestCall *call;
-    GSimpleAsyncResult *result;
+    GTask *task;
 
     g_return_if_fail(OVIRT_IS_RESOURCE(resource));
     g_return_if_fail(OVIRT_IS_PROXY(proxy));
     g_return_if_fail((cancellable == NULL) || G_IS_CANCELLABLE(cancellable));
 
-    result = g_simple_async_result_new(G_OBJECT(resource), callback,
-                                       user_data,
-                                       ovirt_resource_refresh_async);
+    task = g_task_new(G_OBJECT(resource),
+                      cancellable,
+                      callback,
+                      user_data);
     call = ovirt_resource_rest_call_new(REST_PROXY(proxy),
                                         OVIRT_RESOURCE(resource));
     /* FIXME: to set or not to set ?? */
     rest_proxy_call_add_header(REST_PROXY_CALL(call),
                                "All-Content", "true");
     rest_proxy_call_set_method(REST_PROXY_CALL(call), "GET");
-    ovirt_rest_call_async(OVIRT_REST_CALL(call), result, cancellable,
+    ovirt_rest_call_async(OVIRT_REST_CALL(call), task, cancellable,
                           ovirt_resource_refresh_async_cb, resource,
                           NULL);
     g_object_unref(G_OBJECT(call));
@@ -919,9 +908,7 @@ gboolean ovirt_resource_refresh_finish(OvirtResource *resource,
                                        GError **err)
 {
     g_return_val_if_fail(OVIRT_IS_RESOURCE(resource), FALSE);
-    g_return_val_if_fail(g_simple_async_result_is_valid(result,
-                                                        G_OBJECT(resource),
-                                                        ovirt_resource_refresh_async),
+    g_return_val_if_fail(g_task_is_valid(G_TASK(result), G_OBJECT(resource)),
                          FALSE);
 
     return ovirt_rest_call_finish(result, err);
@@ -955,10 +942,10 @@ void ovirt_resource_add_rest_params(OvirtResource *resource,
     OvirtResourceClass *klass;
 
     g_return_if_fail(OVIRT_IS_RESOURCE(resource));
-    g_return_if_fail(OVIRT_IS_RESOURCE_REST_CALL(call));
+    g_return_if_fail(OVIRT_IS_REST_CALL(call));
 
     klass = OVIRT_RESOURCE_GET_CLASS(resource);
-    if (klass->to_xml != NULL)
+    if (klass->add_rest_params != NULL)
         klass->add_rest_params(resource, call);
 }
 
@@ -1035,20 +1022,21 @@ void ovirt_resource_delete_async(OvirtResource *resource,
                                  GAsyncReadyCallback callback,
                                  gpointer user_data)
 {
-    GSimpleAsyncResult *result;
+    GTask *task;
     OvirtResourceRestCall *call;
 
     g_return_if_fail(OVIRT_IS_RESOURCE(resource));
     g_return_if_fail(OVIRT_IS_PROXY(proxy));
     g_return_if_fail((cancellable == NULL) || G_IS_CANCELLABLE(cancellable));
 
-    result = g_simple_async_result_new(G_OBJECT(resource), callback,
-                                       user_data,
-                                       ovirt_resource_delete_async);
+    task = g_task_new(G_OBJECT(resource),
+                      cancellable,
+                      callback,
+                      user_data);
 
     call = ovirt_resource_rest_call_new(REST_PROXY(proxy), resource);
     rest_proxy_call_set_method(REST_PROXY_CALL(call), "DELETE");
-    ovirt_rest_call_async(OVIRT_REST_CALL(call), result, cancellable,
+    ovirt_rest_call_async(OVIRT_REST_CALL(call), task, cancellable,
                           ovirt_resource_delete_async_cb,
                           g_object_ref(resource), g_object_unref);
     g_object_unref(G_OBJECT(call));
@@ -1069,8 +1057,7 @@ gboolean ovirt_resource_delete_finish(OvirtResource *resource,
                                       GError **err)
 {
     g_return_val_if_fail(OVIRT_IS_RESOURCE(resource), FALSE);
-    g_return_val_if_fail(g_simple_async_result_is_valid(result, G_OBJECT(resource),
-                                                        ovirt_resource_delete_async),
+    g_return_val_if_fail(g_task_is_valid(G_TASK(result), G_OBJECT(resource)),
                          FALSE);
     g_return_val_if_fail((err == NULL) || (*err == NULL), FALSE);
 
